import argparse
from argparse import ArgumentParser
from cli.Match import Match
from cli.MatchSegment import MatchSegment
from lotte.Lotte import Lotte
import json


class IntCheckAction(argparse.Action):

    def __call__(self, parser, namespace, values, option_string=None):

        if option_string == '--min-match-length':
            if int(values) < 3:
                parser.error("Minimum value for {0} is 3".format(option_string))
        elif option_string == '--look-back-limit':
            if int(values) < 0:
                parser.error("{0} must be positive".format(option_string))
        elif option_string == '--look-ahead-limit':
            if int(values) < 0:
                parser.error("{0} must be positive".format(option_string))
        elif option_string == '--max-merge-distance':
            if int(values) < 0:
                parser.error("{0} must be positive".format(option_string))
        elif option_string == '--max-merge-ellipse-distance':
            if int(values) < 0:
                parser.error("{0} must be positive".format(option_string))

        setattr(namespace, self.dest, values)


def json_encoder(obj):
    if isinstance(obj, MatchSegment):
        result_dict = obj.__dict__

        if not result_dict['text']:
            del result_dict['text']

        return result_dict

    return obj.__dict__


def main():
    argument_parser = ArgumentParser(description="LotteCLI allows the user to find quotations in two texts, a source"
                                                 "text and a target text. If known, the source text should be the one"
                                                 "that is quoted by the target text. This allows the algorithm to"
                                                 "handle things like ellipses in quotations.")

    argument_parser.add_argument("source_file_path", nargs=1, metavar="source-file-path",
                                 help="Path to the source text file")
    argument_parser.add_argument("target_file_path", nargs=1, metavar="target-file-path",
                                 help="Path to the target text file")
    argument_parser.add_argument('--text', dest="export_text", default=True, action=argparse.BooleanOptionalAction,
                                 help="Include matched text in the returned data structure")
    argument_parser.add_argument('--output-type', choices=['json', 'text'], dest="output_type", default="json",
                                 help="The output type")
    argument_parser.add_argument('--min-match-length', dest="min_match_length", action=IntCheckAction,
                                 default=5, type=int, help="The length of the shortest match (>= 3, default: 5)")

    argument_parser.add_argument('--look-back-limit', dest="look_back_limit", action=IntCheckAction,
                                 default=10, type=int, help="The number of tokens to skip when looking backwards"
                                                            " (>= 0, default: 10), (Very rarely needed)")
    argument_parser.add_argument('--look-ahead-limit', dest="look_ahead_limit", action=IntCheckAction,
                                 default=3, type=int, help="The number of tokens to skip when looking ahead"
                                                           " (>= 0, default: 3)")
    argument_parser.add_argument('--max-merge-distance', dest="max_merge_distance", action=IntCheckAction,
                                 default=2, type=int, help="The maximum distance in tokens between to matches"
                                                           "considered for merging (>= 0, default: 2)")
    argument_parser.add_argument('--max-merge-ellipse-distance', dest="max_merge_ellipse_distance",
                                 action=IntCheckAction, default=10, type=int,
                                 help="The maximum distance in tokens between two matches considered for merging where"
                                      "the target text contains an ellipses between the matches (>= 0, default: 10)")

    args = argument_parser.parse_args()

    file_1_path = args.source_file_path[0]
    file_2_path = args.target_file_path[0]
    export_text = args.export_text
    output_type = args.output_type
    min_match_length = args.min_match_length
    look_ahead_limit = args.look_ahead_limit
    look_back_limit = args.look_back_limit
    max_merge_distance = args.max_merge_distance
    max_merge_ellipse_distance = args.max_merge_ellipse_distance

    with open(file_1_path, 'r') as file_1_file:
        file_1_content = file_1_file.read()

    with open(file_2_path, 'r') as file_2_file:
        file_2_content = file_2_file.read()

    lotte = Lotte(min_match_length, look_back_limit, look_ahead_limit, max_merge_distance, max_merge_ellipse_distance)
    matches = lotte.compare(file_1_content, file_2_content)

    output_matches = []

    for match in matches:
        source_match_segment = match.source_match_segment
        target_match_segment = match.target_match_segment

        source_output_match_segment = MatchSegment(source_match_segment.character_start_pos,
                                                   source_match_segment.character_end_pos)

        target_output_match_segment = MatchSegment(target_match_segment.character_start_pos,
                                                   target_match_segment.character_end_pos)

        if export_text:
            source_text = file_1_content[source_match_segment.character_start_pos:
                                         source_match_segment.character_end_pos]
            target_text = file_2_content[target_match_segment.character_start_pos:
                                         target_match_segment.character_end_pos]

            source_output_match_segment.text = source_text
            target_output_match_segment.text = target_text

        output_match = Match(source_output_match_segment, target_output_match_segment)
        output_matches.append(output_match)

    if output_type == 'json':
        json_string = json.dumps(output_matches, default=json_encoder)
        print(json_string)
    else:
        result = ''

        for match in output_matches:
            match_literature = match.source_match_segment
            match_scientific = match.target_match_segment

            result += f'\n\n{match_literature.character_start_pos}\t{match_literature.character_end_pos}'

            if export_text:
                result += f'\t{match_literature.text}'

            result += f'\n{match_scientific.character_start_pos}\t{match_scientific.character_end_pos}'

            if export_text:
                result += f'\t{match_scientific.text}'

        print(result)


if __name__ == '__main__':
    main()
